//+------------------------------------------------------------------+
//|                                                Trade Manager.mq4 |
//|                        Copyright 2018, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2018, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict

#resource "\\indicators\\Heiken Ashi.ex4"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct Trade
  {
   datetime          ORDER_TIME;
   int               TKT_NUM;
   int               TYPE;
   string            SYMBOL;
   double            PRICE;
   double            VOLUME;
   double            STOP_LOSS;
   double            TAKE_PROFIT;
   datetime          CLOSE_TIME;
   double            CLOSE_PRICE;
   double            SWAP;
   double            COMMISION;
   double            TAX;
   double            PROFIT;
   string            COMMENT;

  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum TRADE_RANK
  {
   HIGHEST,
   LOWEST
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum TRAILING_MODE
  {
   PERCENTAGR,
   PIPS
  };

enum CONSOLODATION_MODE
  {
    LEVELS ,
    RANGE 
    
  };
input ENUM_TIMEFRAMES   MainTF=PERIOD_M15;
input ENUM_TIMEFRAMES   SubTF=PERIOD_M5;
input int    SignalCandles=3;
input string NormalNotification="Type Your Message ";
input string SpecialNotfication="Type Your Special ";
input bool  PopupAlert=true;
input bool  PushNotification=true;
input bool  EmailAlert=true;

input string IndicatorsSettings="======Indicators Settings=====";

input string _RSISettings="===== RSI ====";
input int _RSIPeriod=10;
input ENUM_APPLIED_PRICE _RSIPrice=PRICE_CLOSE;
input double _RSIBullish=55;
input double _RSIBearish=45;

input string _MASettings="==== Moving Average ====";
input int _MAPeriod=5;
input int _MAShift=2;
input ENUM_MA_METHOD _MAMethod=MODE_EMA;
input ENUM_APPLIED_PRICE _MAPrice=PRICE_TYPICAL;

input string _STOSettings=" ===== Stoch astic Settings ====";
input int   _STOKPeriod= 11;
input int   _STODPeriod=3;
input int   _STOSlowing=3;
input ENUM_STO_PRICE _STOPrice=STO_LOWHIGH;
input ENUM_MA_METHOD _STOMAMethod=MODE_SMA;
input bool _STOCON = true ; // Detect Stochastic consolidation
input int   _STOConPeriod=10 ; // Consolidation Period
input CONSOLODATION_MODE _STOConMode=LEVELS ;
input double   _STOUpperLimit_Short=100 ; 
input double   _STOLowerLimit_Short=70 ;
input double   _STOUpperLimit_Long=40 ; 
input double   _STOLowerLimit_Long=0 ;
input double   _STORangeChange=0.40 ;

input string _BreakEvenSettings=" ====== BreakEven Settings ====== ";
input int _Breakeven_Start=25;
input int _Breakeven_Step=5;

input string _TrailingStopSettings=" ====== Trailing Stop Settings ===== ";
input TRAILING_MODE _TrailingStopMode=PERCENTAGR;
input int   _TrailingStopStart=100;
input double   _TrailingStopPect=0.80;
input int      _TrailingStopPips=5;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct SIGNALINFO
  {
   datetime          signalTime;
   int               signalIndex;
   int               timeframe;
   int               type;
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct HKCANDLE
  {
   datetime          time;
   double            open;
   double            close;
  };

int _curMTFBars=0;
int _curSTFBars=0;
int pip=1;
int stoplevel;
int totalBuys=0;
int totalSells=0;
int totalTrades=0;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   pip=Digits==5 || Digits==3?10:1;
   stoplevel=(int)MarketInfo(Symbol(),MODE_STOPLEVEL);

   if(MainTF<SubTF)
     {
      MessageBox("Main Timeframe can't be lower than Sub Time Frame");
      return(INIT_FAILED);
     }
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---

  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   UpdatePL();
   BreakEven_Func();
   TrailingStop_Func();

   SIGNALINFO _cmsi={-1};
   SIGNALINFO _pmsi={-1};
   SIGNALINFO _cssi={-1};
   SIGNALINFO _pssi={-1};

   int _curms=GetLastMTFSignal(1,_cmsi); // Current Main Signal
   int _prems=GetLastMTFSignal(2,_pmsi); // Pre Main Signal
                                         //int _curss=GetLastSTFSginal(1,_cssi) ; // Surrent Sub Signal
// int _press=GetLastSTFSginal(2,_pssi) ; // Pre Sub Signal

//  Normal Signal Main Time Frame
   if(_curMTFBars!=iBars(Symbol(),MainTF))
     {
      //Alert Normal Bullish Signal
      if(_curms==0 && _prems!=0)
        {
         // Check for Special signal
         int bs=iBarShift(Symbol(),SubTF,iTime(Symbol(),MainTF,1));
         bool specialFound=false;
         for(int i=0;i<=bs;i++)
           {
            int _curss=GetLastSTFSginal(i,_cssi);
            if(_curss==0)
              {
               string msgsub="LONG :"+SpecialNotfication+" @"+TimeToString(TimeCurrent(),TIME_DATE|TIME_MINUTES);
               Print(msgsub);
               if(PopupAlert)
                  Alert(msgsub);
               if(PushNotification)
                  SendNotification(msgsub);
               if(EmailAlert)
                  SendMail(msgsub,msgsub);
               specialFound=true;
               break;
              }
           }
         if(!specialFound)
           {
            string msg="LONG :"+NormalNotification+" @"+TimeToString(TimeCurrent(),TIME_DATE|TIME_MINUTES);
            Print(msg);
            if(PopupAlert)
               Alert(msg);
            if(PushNotification)
               SendNotification(msg);
            if(EmailAlert)
               SendMail(msg,msg);
           }
        }
      // Alert Bearish Signal
      else if(_curms==1 && _prems!=1)
        {
         // Check for Special signal
         int bs=iBarShift(Symbol(),SubTF,iTime(Symbol(),MainTF,1));
         bool specialFound=false;
         for(int i=0;i<=bs;i++)
           {
            int _curss=GetLastSTFSginal(i,_cssi);
            if(_curss==1)
              {
               string msgsub="SHORT :"+SpecialNotfication+" @"+TimeToString(TimeCurrent(),TIME_DATE|TIME_MINUTES);
               Print(msgsub);
               if(PopupAlert)
                  Alert(msgsub);
               if(PushNotification)
                  SendNotification(msgsub);
               if(EmailAlert)
                  SendMail(msgsub,msgsub);
               specialFound=true;
               break;
              }
           }
         if(!specialFound)
           {
            string msg="SHORT :"+NormalNotification+" @"+TimeToString(TimeCurrent(),TIME_DATE|TIME_MINUTES);
            Print(msg);
            if(PopupAlert)
               Alert(msg);
            if(PushNotification)
               SendNotification(msg);
            if(EmailAlert)
               SendMail(msg,msg);
           }
        }
      _curMTFBars=iBars(Symbol(),MainTF);
     }

  }
//-------------------------------------------------------------------+
void UpdatePL()
  {
   if(GetOrdersCount()>0)
     {
      if(!GlobalVariableCheck("TM_"+Symbol()+"_PL"))
        {
         GlobalVariableSet("TM_"+Symbol()+"_PL",Get_PL());
        }
      else
        {
         double curPL=Get_PL();
         double prePL=GlobalVariableGet("TM_"+Symbol()+"_PL");
         if(curPL>prePL)
            GlobalVariableSet("TM_"+Symbol()+"_PL",curPL);
        }
     }
   else
     {
      GlobalVariableDel("TM_"+Symbol()+"_PL");
     }
  }
//+------------------------------------------------------------------+
int GetHKChangeIndex(int timeframe,int index)
  {

   for(int i=index;i<iBars(Symbol(),timeframe)-1;i++)
     {
      HKCANDLE cur=HK(i,timeframe);
      HKCANDLE pre=HK(i+1,timeframe);
      if((cur.close>=cur.open && pre.close<pre.open) || (cur.close<cur.open && pre.close>=pre.open))
         return i;
     }
   return -1;
  }
//-------------------------------------------------------------------+
HKCANDLE HK(int i,int timeframe)
  {
   HKCANDLE hk={0};
   hk.time=iTime(Symbol(),timeframe,i);
   hk.close=iCustom(Symbol(),timeframe,"::Indicators\\Heiken Ashi.ex4",3,i);
   hk.open=iCustom(Symbol(),timeframe,"::Indicators\\Heiken Ashi.ex4",2,i);
   return hk;
  }
//--------------------------------------------------------------------+
double RSI(int i,int timeframe)
  {
   double rsi=iRSI(Symbol(),timeframe,_RSIPeriod,_RSIPrice,i);
   return rsi;
  }
//--------------------------------------------------------------------+
double STOCH(int i,int timeframe,int mode)
  {
   double val=iStochastic(Symbol(),timeframe,_STOKPeriod,_STODPeriod,_STOSlowing,_STOMAMethod,_STOPrice,mode,i);
   return val;
  }
//---------------------------------------------------------------------+
double MA(int i,int timeframe)
  {
   double ma=iMA(Symbol(),timeframe,_MAPeriod,_MAShift,_MAMethod,_MAPrice,i);
   return ma;
  }
//----------------------------------------------------------------------+
bool IsStoConsolidat(int i , int timeframe,int type){
   if(!_STOCON)
     {
       return true ;
     }
    
   
    if(_STOConMode==LEVELS)
    {
       double upper=type==OP_BUY?_STOUpperLimit_Long:_STOUpperLimit_Short ;
       double lower=type==OP_BUY?_STOLowerLimit_Long:_STOLowerLimit_Short ;
      for(int index=i;index<i+_STOConPeriod;index++)
      {
        double _cursto=iStochastic(Symbol(),timeframe,_STOKPeriod,_STODPeriod,_STOSlowing,_STOMAMethod,_STOPrice,MODE_MAIN,index) ;
        bool state=_cursto>=lower && _cursto<=upper;
        if(state)continue ;
        else
         return false ;
      }
    }
    else if(_STOConMode==RANGE)
    {
      double firstval=iStochastic(Symbol(),MainTF,_STOKPeriod,_STODPeriod,_STOSlowing,_STOMAMethod,_STOPrice,MODE_MAIN,i) ;
      double upper=NormalizeDouble(firstval*(1+_STORangeChange),2) ;
      double lower=NormalizeDouble(firstval*(1-_STORangeChange),2) ;
      for(int index=i+1;index<i+_STOConPeriod;index++)
        {
         double curval=iStochastic(Symbol(),MainTF,_STOKPeriod,_STODPeriod,_STOSlowing,_STOMAMethod,_STOPrice,MODE_MAIN,index) ;
         if(curval<=upper && curval>=lower)
          continue ;
         else
          return false ;
        }
      
    }
    
    return true ;
}
//----------------------------------------------------------------------+
int GetOrdersCount()
  {
   int cnt=0;
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS) && OrderSymbol()==Symbol() && OrderType()<2)
        {
         cnt++;
        }
     }
   return cnt;

  }
//------------------------------------------------------------------------------+
int GetOrdersCount(int type)
  {
   int cnt=0;
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS) && OrderSymbol()==Symbol() && OrderType()==type)
        {
         cnt++;
        }
     }
   return cnt;

  }
//----------------------------------------------------------------------+
Trade GetTrade(TRADE_RANK trade_rank,int type)
  {
   Trade t={0};
   Trade AllTrades[];
   for(int i=0;i<OrdersTotal();i++)
     {
      Trade _curt={0};
      if(OrderSelect(i,SELECT_BY_POS) && OrderSymbol()==Symbol() && OrderType()==type)
        {
         ArrayResize(AllTrades,i+1);
         _curt.ORDER_TIME=OrderOpenTime();
         _curt.TKT_NUM=OrderTicket();
         _curt.TYPE=OrderType();
         _curt.SYMBOL=OrderSymbol();
         _curt.PRICE=OrderOpenPrice();
         _curt.VOLUME=OrderLots();
         _curt.STOP_LOSS=OrderStopLoss();
         _curt.TAKE_PROFIT=OrderTakeProfit();
         _curt.CLOSE_PRICE=OrderClosePrice();
         _curt.CLOSE_TIME=OrderCloseTime();
         _curt.SWAP=OrderSwap();
         _curt.TAX=0;
         _curt.COMMISION=OrderCommission();
         _curt.PROFIT=OrderProfit();
         _curt.COMMENT=OrderComment();
         AllTrades[i]=_curt;
        }
     }

   t=AllTrades[0];

   for(int i=0;i<ArraySize(AllTrades);i++)
     {
      if(trade_rank==HIGHEST)
        {
         if(AllTrades[i].PRICE>t.PRICE)
            t=AllTrades[i];
         else
            continue;
        }
      else if(trade_rank==LOWEST)
        {
         if(AllTrades[i].PRICE<t.PRICE)
            t=AllTrades[i];
         else
            continue;
        }
     }
   return t;
  }
///------------------------------------------------------------------------------+
double Get_PL()
  {
   double pl=0;
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS) && OrderSymbol()==Symbol() && OrderType()<2)
        {
         double current=OrderType()==OP_BUY?NormalizeDouble(OrderClosePrice()-OrderOpenPrice(),Digits):NormalizeDouble(OrderOpenPrice()-OrderClosePrice(),Digits);
         pl+=current;
        }
     }
   return pl;
  }
//------------------------------------------------------------------------------+
double Get_PL(int type)
  {
   double pl=0;
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS) && OrderSymbol()==Symbol() && OrderType()==type)
        {
         double current=OrderType()==OP_BUY?NormalizeDouble(OrderClosePrice()-OrderOpenPrice(),Digits):NormalizeDouble(OrderOpenPrice()-OrderClosePrice(),Digits);
         pl+=current;
        }
     }
   return pl;
  }
//-----------------------------------------------------------------------+
void ModifyOrders(int type,double newstop)
  {
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS) && OrderSymbol()==Symbol() && OrderType()==type)
        {
         bool modified=OrderModify(OrderTicket(),OrderOpenPrice(),newstop,OrderTakeProfit(),0,clrNONE);
        }
     }
  }
//-----------------------------------------------------------------------+
void CloseOrders()
  {
   for(int i=OrdersTotal()-1;i>=0;i--)
     {
      if(OrderSelect(i,SELECT_BY_POS) && OrderSymbol()==Symbol() && OrderType()<2)
        {
         bool closed=OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),3*pip,clrNONE);
        }
     }
  }
//-----------------------------------------------------------------------+
void CloseOrders(int type)
  {
   for(int i=OrdersTotal()-1;i>=0;i--)
     {
      if(OrderSelect(i,SELECT_BY_POS) && OrderSymbol()==Symbol() && OrderType()==type)
        {
         bool closed=OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),3*pip,clrNONE);
        }
     }
  }
//-----------------------------------------------------------------------+
int GetLastMTFSignal(int ind,SIGNALINFO &si)
  {
   int index=GetHKChangeIndex(MainTF,ind);
   if(index>SignalCandles || index<0) return -1;

   HKCANDLE hk=HK(index,MainTF);

   HKCANDLE curhk=HK(ind,MainTF);
   double _ma=MA(ind,MainTF);
   double _rsi=RSI(ind,MainTF);
   double _stomaincur=STOCH(ind,MainTF,MODE_MAIN);
   double _stosingalcur=STOCH(ind,MainTF,MODE_SIGNAL);
   double _stomainpre=STOCH(ind+1,MainTF,MODE_MAIN);
   double _stosingalpre=STOCH(ind+1,MainTF,MODE_SIGNAL);
   if(hk.close>=hk.open && curhk.close>=curhk.open && curhk.close>_ma && _rsi>=_RSIBullish && _stomaincur>_stosingalcur && _stomaincur>_stomainpre && _stomaincur<80 && IsStoConsolidat(ind+1,MainTF,OP_BUY))
     {
      si.signalIndex=ind;
      si.signalTime=iTime(Symbol(),MainTF,ind);
      si.timeframe=MainTF;
      si.type=0;
      return 0;
     }
   else if(hk.close<hk.open && curhk.close<curhk.open && curhk.close<_ma && _rsi<_RSIBearish && _stomaincur<_stosingalcur && _stomaincur<_stomainpre && _stomaincur>20 && IsStoConsolidat(ind+1,MainTF,OP_SELL))
     {
      si.signalIndex=ind;
      si.signalTime=iTime(Symbol(),MainTF,ind);
      si.timeframe=MainTF;
      si.type=1;
      return 1;
     }

   return -1;
  }
//------------------------------------------------------------------------------+
int GetLastSTFSginal(int index,SIGNALINFO &si)
  {
   HKCANDLE hk=HK(index,SubTF);
   double _ma=MA(index,SubTF);
   double _rsi=RSI(index,SubTF);
   double _stomaincur=STOCH(index,SubTF,MODE_MAIN);
   double _stosingalcur=STOCH(index,SubTF,MODE_SIGNAL);
   double _stomainpre=STOCH(index+1,SubTF,MODE_MAIN);
   double _stosingalpre=STOCH(index+1,SubTF,MODE_SIGNAL);
   if(hk.close>=hk.open && hk.close>_ma && _rsi>=_RSIBullish && _stomaincur>_stosingalcur && _stomaincur>_stomainpre && _stomaincur<80)
     {
      si.signalIndex=index;
      si.signalTime=iTime(Symbol(),SubTF,index);
      si.timeframe=SubTF;
      si.type=0;
      return 0;
     }
   else if(hk.close<hk.open && hk.close<_ma && _rsi<_RSIBearish && _stomaincur<_stosingalcur && _stomaincur<_stomainpre && _stomaincur>20)
     {
      si.signalIndex=index;
      si.signalTime=iTime(Symbol(),SubTF,index);
      si.timeframe=SubTF;
      si.type=1;
      return 1;
     }
   return -1;
  }
//-------------------------------------------------------------------------------+
void BreakEven_Func()
  {
   if(GetOrdersCount()==0 || (_Breakeven_Start-_Breakeven_Step)*pip<stoplevel || _Breakeven_Start-_Breakeven_Step<=0  || _Breakeven_Start<=0 ) return;

// Breakeven for buy
   if(GetOrdersCount(OP_BUY)>0)
     {
      Trade _highestT=GetTrade(HIGHEST,OP_BUY);
      double pl=NormalizeDouble(_highestT.CLOSE_PRICE-_highestT.PRICE,Digits);

      if(pl>=_Breakeven_Start*pip*Point)
        {
         double newstop=NormalizeDouble(_highestT.PRICE+_Breakeven_Step*pip*Point,Digits);
         if(newstop>_highestT.STOP_LOSS)
           {
            ModifyOrders(OP_BUY,newstop);
           }

        }
     }
// Breakeven for Sell
   if(GetOrdersCount(OP_SELL)>0)
     {
      Trade _lowestT=GetTrade(LOWEST,OP_SELL);
      double pl=NormalizeDouble(_lowestT.PRICE-_lowestT.CLOSE_PRICE,Digits);

      if(pl>=_Breakeven_Start*pip*Point)
        {
         double newstop=NormalizeDouble(_lowestT.PRICE-_Breakeven_Step*pip*Point,Digits);
         if(newstop<_lowestT.STOP_LOSS || _lowestT.STOP_LOSS<=0)
           {
            ModifyOrders(OP_SELL,newstop);
           }

        }
     }
  }
//-----------------------------------------------------------------------------------+
void TrailingStop_Func()
  {
   if(_TrailingStopStart<=0 || GetOrdersCount()==0)return;

   double _totalpl=Get_PL();
   double _buyspl=Get_PL(OP_BUY);
   double _sellpl=Get_PL(OP_SELL);

   if(_TrailingStopMode==PERCENTAGR)
     {
      if(_TrailingStopPect<=0)return;
      double highestProfit=GlobalVariableGet("TM_"+Symbol()+"_PL");
      if(_totalpl<=NormalizeDouble(highestProfit*_TrailingStopPect,2) && highestProfit>=_TrailingStopStart*pip*Point)
        {
         CloseOrders();
        }
     }
   else if(_TrailingStopMode==PIPS)
     {
      if(_TrailingStopPips*pip<stoplevel || _TrailingStopPips<=0)return;

      // Trail Buy ORders
      if(_buyspl>=_TrailingStopStart*pip*Point)
        {
         double newstop=NormalizeDouble(Bid-_TrailingStopPips*pip*Point,Digits);
         for(int i=0;i<OrdersTotal();i++)
           {
            if(OrderSelect(i,SELECT_BY_POS) && OrderSymbol()==Symbol() && OrderType()==OP_BUY && newstop>OrderStopLoss())
              {
               bool modified=OrderModify(OrderTicket(),OrderOpenPrice(),newstop,OrderTakeProfit(),0,clrNONE);
              }
           }
        }

      // Trail Sell Orders
      if(_sellpl>=_TrailingStopStart*pip*Point)
        {
         double newstop=NormalizeDouble(Ask+_TrailingStopPips*pip*Point,Digits);
         for(int i=0;i<OrdersTotal();i++)
           {
            if(OrderSelect(i,SELECT_BY_POS) && OrderSymbol()==Symbol() && OrderType()==OP_SELL && (newstop<OrderStopLoss() || OrderStopLoss()==0))
              {
               bool modified=OrderModify(OrderTicket(),OrderOpenPrice(),newstop,OrderTakeProfit(),0,clrNONE);
              }
           }
        }
     }
  }
//+------------------------------------------------------------------+
